@@ -121,6 +121,8 @@ extern sexp* rlang_new_weakref(sexp*, sexp*, sexp*, sexp*);
 extern sexp* rlang_wref_key(sexp*);
 extern sexp* rlang_wref_value(sexp*);
 extern sexp* rlang_is_weakref(sexp*);
+extern sexp* rlang_find_var(sexp*, sexp*);
+extern sexp* rlang_env_bind_list(sexp*, sexp*, sexp*);
 
 // Library initialisation defined below
 sexp* rlang_library_load();
@@ -281,6 +283,8 @@ static const r_callable r_callables[] = {
   {"rlang_wref_key",                    (r_fn_ptr) &rlang_wref_key, 1},
   {"rlang_wref_value",                  (r_fn_ptr) &rlang_wref_value, 1},
   {"rlang_is_weakref",                  (r_fn_ptr) &rlang_is_weakref, 1},
+  {"rlang_find_var",                    (r_fn_ptr) &rlang_find_var, 2},
+  {"rlang_env_bind_list",               (r_fn_ptr) &rlang_env_bind_list, 3},
   {NULL, NULL, 0}
 };
 
