@@ -24,5 +24,9 @@ run_script <- function(file, envvars = chr()) {
 }
 
 local_methods <- function(..., .frame = caller_env()) {
-  scoped_bindings(..., .env = global_env(), .frame = .frame)
+  local_bindings(..., .env = global_env(), .frame = .frame)
+}
+with_methods <- function(.expr, ...) {
+  local_methods(...)
+  .expr
 }
