@@ -12,11 +12,11 @@ test_that("locale setters report old locale", {
 })
 
 old_digits <- getOption("digits")
-test_that("scoped_options() sets options", {
-  old <- scoped_options(digits = 2L)
+test_that("local_options() sets options", {
+  old <- local_options(digits = 2L)
   expect_identical(old$digits, old_digits)
   expect_identical(getOption("digits"), 2L)
 })
-test_that("scoped_options() restores options", {
+test_that("local_options() restores options", {
   expect_identical(getOption("digits"), old_digits)
 })
